%% Spatio-Spectral Vizualisation (4D)
% Antony Schutz - Lagrange/Fizeau 2012
function Affiche4D(data,thresh,colormaptype,fact)

switch nargin
    case 1, thresh = 0.05; colormaptype= 'hot'; fact = 1;
    case 2, colormaptype= 'hot'; fact = 1;
    case 3, fact = 1;
        
end

% Number of elements
[m,n,l]     = size(data);
% Cube's size
edgesize    = [1,1,1].*fact;
% Opacity depends on the distance to the center of the cube and intensity
% Color depends on the absolutve value of the point
% Pre processing, position and intensity vectorization
[X,Y,Z,V]   = Make3dTo4dVect(data,thresh);                   % thresholding
NCube       = length(V);
% Intensity normalisation of color (0 is white 1 is black)
I           = abs(V)./max(abs(V));
% Center's position (rounded) and distance max
mx          = round(m./2);
my          = round(n./2);
mz          = round(l./2);
Maxdist     = sqrt( (m-mx).^2 + (n-my).^2 + (l-mz).^2);    
% Cube's plot depends on intensity, position, and transparancy
im = figure;
figure(im);
for indice  = 1 : NCube 
    x           = X(indice).*fact;
    y           = Y(indice).*fact;
    z           = Z(indice).*fact;
    Rho         = sqrt( (x-mx).^2 + (y-my).^2 + (z-mz).^2);         
    origin      = [x,y,z];
    alpha       = (1-Rho./Maxdist).*I(indice);
    color       = color_index(I(indice),colormaptype);
    plotcube(edgesize,origin,alpha,color);
end
% 2D
vectX       = min(X):max(X);
vectY       = min(Y):max(Y);
vectZ       = min(Z):max(Z);
edgesize2d  = [1,1,.001];
Max3dx      = max(max(sum(data,1)));
Max3dy      = max(max(sum(data,2)));
Max3dz      = max(max(sum(data,3)));
alpha2d     = 0.5;
% Contour (XY) plot
for vx = vectX
for vy = vectY

    vI      = sum(data(vx,vy,:))./Max3dz;
    if vI>0
    origin  = [vx,vy,0];
    color   = color_index(vI,colormaptype);    
    plotcube(edgesize2d,origin,alpha2d,color);
    end
end    
end
% Contour (XZ) plot
edgesize2d  = [1,.001,1];
for vx = vectX
for vz = vectZ
    vI      = sum(data(vx,:,vz))./Max3dy;
    if vI>0
    origin  = [vx,n+1,vz];
    color   = color_index(vI,colormaptype);
    plotcube(edgesize2d,origin,alpha2d,color);
    end
end    
end
% Contour (YZ) plot
edgesize2d  = [.001,1,1];
for vz = vectZ
for vy = vectY
    vI      = sum(data(:,vy,vz))./Max3dx;
    if vI>0
    origin  = [m+1,vy,vz];
    color   = color_index(vI,colormaptype);
    plotcube(edgesize2d,origin,alpha2d,color);
    end
end    
end
axis equal; grid on;
figure(im);
% Axis drawing

xlabel(' FoV (mas) ','color','k','fontsize',18); 
ylabel(' FoV (mas) ','color','k','fontsize',18); 
zlabel(' $\lambda$','color','k','fontsize',18,'Rotation',0,'interpreter','latex');
% Control Button
ButXYZ      = uicontrol(gcf,'Style','pushbutton','String','XY lambda','Callback','view(3)');
POSITI      = get(ButXYZ,'Position');
left        = POSITI(1);
bottom      = POSITI(2);
width       = POSITI(3);
height      = POSITI(4);
set(ButXYZ,'Position',[left bottom+height*3 width height]);
ButXZ       = uicontrol(gcf,'Style','pushbutton','String','X lambda','Position',[left bottom+height*2   width height],'Callback','view(0,00)');
ButYZ       = uicontrol(gcf,'Style','pushbutton','String','Y lambda','Position',[left bottom+height*1   width height],'Callback','view(-90,0)');
ButXY       = uicontrol(gcf,'Style','pushbutton','String','XY'      ,'Position',[left bottom+height*0 width height],'Callback','view(0,90)');
% set(im,'WindowStyle','normal')
set(im,'Color','w','MenuBar','figure','ToolBar','figure');
set(gca, 'CLim', 1-fliplr(caxis));
hcbar = colorbar('location','EastOutside');

hold on; 
line([1 m+1]  ,[1+n 1+n],[0 0]  ,'Color','k','LineWidth',2); 
line([m+1 m+1],[1 1+n]  ,[0 0]  ,'Color','k','LineWidth',2); 
line([m+1 m+1],[1+n 1+n],[0 1+l],'Color','k','LineWidth',2); 
hold off;

title(' $(X, \ Y, \ \lambda)$ ','fontsize',18,'interpreter','latex');
set(gcf,'Name',' 4D Vizualization');
set(gca,'fontsize',18)
end


function XYZ = plotcube(varargin)
% Replace default input arguments by input values
inArgs(1:nargin) = varargin;

% Create all variables
[edges,origin,alpha,clr] = deal(inArgs{:});

XYZ = { ...
  [0 0 0 0]  [0 0 1 1]  [0 1 1 0] ; ...
  [1 1 1 1]  [0 0 1 1]  [0 1 1 0] ; ...
  [0 1 1 0]  [0 0 0 0]  [0 0 1 1] ; ...
  [0 1 1 0]  [1 1 1 1]  [0 0 1 1] ; ...
  [0 1 1 0]  [0 0 1 1]  [0 0 0 0] ; ...
  [0 1 1 0]  [0 0 1 1]  [1 1 1 1]   ...
  };

XYZ = mat2cell(cellfun(@(x,y,z) x*y+z,XYZ, ...
    repmat(mat2cell(edges ,1,[1 1 1]),6,1), ...
    repmat(mat2cell(origin,1,[1 1 1]),6,1), ...
    'UniformOutput',false),6,[1 1 1]);


 ha=cellfun(@patch,XYZ{1},XYZ{2},XYZ{3},...
  repmat({clr},6,1),...
  repmat({'FaceAlpha'},6,1),...
  repmat({alpha},6,1),...
  repmat({'EdgeColor'},6,1),...
  repmat({clr},6,1),...
  repmat({'LineStyle'},6,1),...  
  repmat({':'},6,1)...
  );

view(3);
end

function [x,y,z,v] = Make3dTo4dVect(data,thresh)

% x first dimension e.g width
% y secon dimension e.g high
% z third dimension e.g deep
% v value of data on [x,y,z]

[m,n,l] = size(data);
Max     = max(data(:));
x       = zeros(m*n*l,1);
y       = x;
z       = x;
v       = x;
p       = 0;
for xx  = 1 : m
for yy  = 1 : n
for zz  = 1 : l
    if (data(xx,yy,zz)/Max)>thresh
    p   = p+1;
    x(p)= xx;
    y(p)= yy;
    z(p)= zz;
    v(p)= data(xx,yy,zz);
    end
end
end
end

x(x==0)=[];
y(y==0)=[];
z(z==0)=[];
v(v==0)=[];
end

function DefColor = color_index(value,ColormapType)
% value is between 0 and 1
% color_index return 1*3 vector of colormap map
% minv minimum value of the data series
% maxv maximum value of the data series
% dv minimum difference value of the data series
% 
% Color sampling

N = 2.^16;

switch ColormapType
    case 'jet',     DefColormap = colormap(flipud(jet(N))); 
    case 'hsv',     DefColormap = colormap(flipud(hsv(N)));    
    case 'hot',     DefColormap = colormap(flipud(hot(N)));
    case 'cool',    DefColormap = colormap(flipud(cool(N)));
    case 'spring',  DefColormap = colormap(flipud(spring(N)));
    case 'summer',  DefColormap = colormap(flipud(summer(N)));
    case 'autumn',  DefColormap = colormap(flipud(autumn(N)));
    case 'winter',  DefColormap = colormap(flipud(winter(N)));
    case 'gray',    DefColormap = colormap(flipud(gray(N)));
    case 'bone',    DefColormap = colormap(flipud(bone(N)));
    case 'copper',  DefColormap = colormap(flipud(copper(N)));
    case 'pink',    DefColormap = colormap(flipud(pink(N)));
    case 'lines',   DefColormap = colormap(flipud(lines(N)));        
end

indice      = max(1,round(value.*N));
DefColor    = DefColormap(indice,:);
end