function yest   = GradphiWMF(options,yt)
% Antony Schutz 2014 - ANR POLCA - CNRS - Lab. J.L.Lagrange
% gradphi using minFunc and weights
% http://www.di.ens.fr/~mschmidt/Software/minFunc.html
addpath(genpath('minFunc'));
if isfield(options,'Xi'   )~=1, error(' No Phases differences data '); end
if isfield(options,'rho_y')~=1, warning(' No ADMM parameter (default rho = 1)');    options.rho_y   = 1;    end
if isfield(options,'beta' )~=1, warning(' No Prox parameter (default beta = 1)');   options.beta    = 1;           end
if isfield(options,'Xierr')~=1, warning(' No Phases differences error bar (default W = 1)');        options.Xierr= ones(size( options.P )); end
phi0            = angle(yt);
% phi0 			= phi0 + pi*(2*rand(size(yt)) -1)/100;

optionopt.Display = 'off';

if isfield(options,'minFuncMaxIter') == 1, if isempty(options.minFuncMaxIter) ~= 1
    optionopt.MaxIter = options.minFuncMaxIter; end, end
if isfield(options,'minFuncMaxFunEvals') == 1, if isempty(options.minFuncMaxFunEvals) ~= 1
    optionopt.MaxFunEvals = options.minFuncMaxFunEvals; end, end

pest = minFunc(@(phi)costgradphi(phi,options,yt),phi0,optionopt);

[~,~,yest,w1,w2] = costgradphi(pest,options,yt);
if isfield(options,'printcrit') == 1 
if options.printcrit == 1
fprintf(' \n PHASE ');
fprintf(' \t w1: %g \t, beta*w1: %g',w1,options.beta*w1);
fprintf('\n \t \t w2: %g \t, rho *w2: %g\n',w2,options.rho_y *w2);
end
end
    
function [cost,grad,yest,w1,w2] = costgradphi(phi,options,yt)
rho_y   = options.rho_y;
H       = options.H;
Xi      = options.Xi;
beta    = options.beta;
K       = options.Xierr;
gam_t   = abs(yt);
phi_t   = angle(yt);

Ek      = phi_t - phi;
gam     = max(0,gam_t.*cos(Ek));
    
dphi    = H*phi - Xi;
grad    = beta*H'*(K.*sin(dphi)) - rho_y.*gam.*gam_t.*sin(Ek);   
yest    = gam.*exp(1i*phi);
    
% w1      = sum(-K.*cos(dphi)) ;
% w2      = sum(abs(yest-yt).^2);
% cost    = abs(beta*w1+.5*rho_y*w2);

w1      = sum((1-cos(dphi)).*K) ;
w2      = sum(abs(yest-yt).^2);
cost    = (beta*w1+rho_y*w2)/2;

% Ek      = phi_t - phi;
% gam     = max(0,gam_t.*cos(Ek));
%     
% dphi    = H*phi - Xi;
% grad    = beta*H'*(sin(dphi)./W) + rho_y.*gam.*gam_t.*sin(-Ek);   
% yest    = gam.*exp(1i*phi);
%     
% w1      = sum((1-cos(dphi))./W) ;
% w2      = sum(abs(yest-yt).^2);
% cost    = (beta*w1+rho_y*w2)/2;