function options = OIDataExtraction(options)
% extract data used in ADMM 2D and 3D
% put the existing data into "options"
datapath    = options.datapath;
CDIR        = dir(fullfile(datapath,'*.fits'));

if isempty(CDIR)==1 , error('No FITS File in folder'); end

Nfil        = length(CDIR);
FileIndex   = 1 : Nfil;

dataornot   = options.dataornot;

VISPHI      = [];
VISPHIERR   = [];
VIS2DATA    = [];
VIS2ERR     = [];
UCOORD      = [];
VCOORD      = [];
T3PHI       = [];
T3PHIERR    = [];
VISAMP      = [];
VISAMPERR   = [];
WAVELENGTH  = [];
lbd         = 0;
V2          = 0;
T3          = 0;
DP          = 0;
m           = 0;
K           = 0;
if isfield(options,'FileIndex') == 1 ,
    FileIndex = options.FileIndex;
end
if isfield(options,'WvlMat') == 1
    Indwvl = [];
    for n = 1 : length(options.WvlMat)
        Indwvl = [Indwvl;options.WvlMat{n}(:)];
    end
    options.Indwvl = sort(Indwvl);
end
for     n   = FileIndex
    dataname    = CDIR(n).name;
    DATA        = ReadOIFITS(dataname,datapath);
    options.DATA{n} = DATA;
    
    % OI_WAVELENGTH
    if isfield(DATA.OI_WAVELENGTH,'EFF_WAVE')
        if isfield(DATA,'OI_WAVELENGTH')
            wvl             = DATA.OI_WAVELENGTH.EFF_WAVE;
            Indwvl          = 1:length(wvl);
            if isfield(options,'Indwvl') ==1,
                Indwvl   = options.Indwvl;
                wvl      = wvl(Indwvl);
            end
            % faire un test pour verifier que tous les fichiers ont les memes
            % bandes spectrales
            lbd             = 1;
        end
        WAVELENGTH = [WAVELENGTH; wvl(:)];
    else
        error(' No Wavelength Information ');
    end
    
    % VIS
    DP          = 0;
    if isfield(DATA,'OI_VIS')
        if isfield(DATA.OI_VIS,'VISPHI')
            VISPHI      = [VISPHI;      DATA.OI_VIS.VISPHI(:,Indwvl)];
            VISPHIERR   = [VISPHIERR;   DATA.OI_VIS.VISPHIERR(:,Indwvl)];
            VISAMP      = [VISAMP;      DATA.OI_VIS.VISAMP(:,Indwvl)];
            VISAMPERR   = [VISAMPERR;   DATA.OI_VIS.VISAMPERR(:,Indwvl)];
            DP          = 1;
        end
     
    else
        warning(' No Diff Phi ');
    end
    
    % VIS 2
    V2          = 0;
    if isfield(DATA.OI_VIS2,'VIS2DATA')
        if isfield(DATA,'OI_VIS2')
            VIS2DATA    = [VIS2DATA;    DATA.OI_VIS2.VIS2DATA(:,Indwvl)];
            VIS2ERR     = [VIS2ERR;     DATA.OI_VIS2.VIS2ERR(:,Indwvl)];
            
            u           = DATA.OI_VIS2.UCOORD;
            v           = DATA.OI_VIS2.VCOORD;
            upoly3D     = repmat(u,1,length(wvl))./repmat(wvl(:)',length(u),1);
            vpoly3D     = repmat(v,1,length(wvl))./repmat(wvl(:)',length(v),1);            
            
            UCOORD      = [UCOORD;upoly3D];
            VCOORD      = [VCOORD;vpoly3D];
            V2          = 1;
        end
    else error(' No Vis 2 ');
    end
    % PHASES CLOSURE
    T3          = 0;
    if isfield(DATA.OI_T3,'T3PHI')
        if isfield(DATA,'OI_T3')
            T3PHI       = [T3PHI;       DATA.OI_T3.T3PHI(:,Indwvl)];
            T3PHIERR    = [T3PHIERR;    DATA.OI_T3.T3PHIERR(:,Indwvl)];
            T3          = 1;
            if V2 == 1
                
                u           = DATA.OI_VIS2.UCOORD;
                u1          = DATA.OI_T3.U1COORD;
                u2          = DATA.OI_T3.U2COORD;
                u3          = u1 + u2;
                utmp        = round(1000*u );
                U1          = round(1000*u1);
                U2          = round(1000*u2);
                U3          = round(1000*u3);
                
                for k       = 1 : length(U1)
                    o           = find(utmp==U1(k)); Closure_index(K+k,1) = m+o;
                    o           = find(utmp==U2(k)); Closure_index(K+k,2) = m+o;
                    o           = find(utmp==U3(k)); Closure_index(K+k,3) = m+o;
                end
                m           = max(Closure_index(:));
                K           = K+k;
            else
                warning('No Vis 2 UV Coord -> No Phases Closure');
            end
        end
    else
        warning(' No Phase Closure ');
    end
    
end
%% options
% WVL
if lbd ==1
    options.wvl  = wvl;
    options.WAVELENGTH = WAVELENGTH;
end
% V2
if V2*lbd == 1
    [options.Nbase,options.Nwvl]= size( UCOORD );

    options.U    = UCOORD;
    options.V    = VCOORD;
end
if V2 == 1;
    options.V2   = VIS2DATA;
    options.V2err= VIS2ERR;
end
if dataornot == 1 
% creation de la matrice H, 2eme version
[H,HT3,HDP] = Ph2PhDiff(Closure_index,options.Nwvl,options.Nbase,T3,DP);
% CLOSURE
ConvCoef    = pi/180;
if T3 == 1
    % verification des unite si en degree mettre en radian
    % l(info se trouve dans DATA.OI_T3.Head faire une recherche 'deg' ou
    % 'rad'
    options.T3   = T3PHI*ConvCoef;
    options.T3err= T3PHIERR*ConvCoef;

%     options.T3cos= EstKapVM(T3PHIERR);
    options.Xi   = options.T3(:);
%     options.Xierr= options.T3cos(:);
    options.H    = HT3;
    options.HT3  = HT3;
end
% DIFFPHI
if DP == 1
    % DiffPhi Creation from Diff
    [DPh,DPerr]  = DPhiProc(VISPHI,VISPHIERR);
    options.DP   = DPh*ConvCoef;
%     options.DPerr= DPerr*ConvCoef;

%     options.DPcos= EstKapVM(DPerr);
    

    options.Xi   = options.DP(:);
%     options.Xierr= options.DPcos(:);
    options.H    = HDP;
    options.HDP  = HDP;
end

if T3==1&&DP==1
    options.H    = H;
    options.Xi   = [options.T3(:);options.DP(:)];
%     options.Xierr= [options.T3cos(:);options.DPcos(:)];

end

if T3==0&&DP==0
    error(' No Phases differences ');
end

end