function options = PAINTER(options)
% Structure Init for ADMM - warning and error checking
warn3D;
%% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - INITIALIZATION
% initialization auxiliary variables
z               = zeros(2*nx*nx,Nwvl);
w               = zeros(nx*nx,Nwvl);
v               = zeros(nx*nx,Nwvl);
r               = zeros(nx*nx,Nwvl-1);
% initialization Lagrange multipliers
tau_tv          = zeros(2*nx*nx,Nwvl);
tau_w           = zeros(nx*nx,Nwvl);
tau_v           = zeros(nx*nx,Nwvl);
tau_r           = zeros(nx*nx,Nwvl-1);
tau_xc          = zeros(Nbase,Nwvl);
tau_pwc         = zeros(Nbase,Nwvl);
tau_xic         = zeros(Nbase,Nwvl);
ys              = zeros(Nbase,Nwvl);

if size(yc,2)  == 1 , yc   = reshape(yc,Nbase,Nwvl);   end
% boucle ADMM
fprintf('\n');
M               = zeros(nx*nx,nx*nx,Nwvl);
Normalize       = zeros(Nwvl,1);
coef            = zeros(Nwvl,1);
    fprintf('\n Number of Wavelength: %d \n', Nwvl);
for n           = 1 : Nwvl, fprintf('|%d', n );
    % matrix inversion pre calculus
    M(:,:,n)        = inv(rho_y*(Fr3D(:,:,n)'*Fr3D(:,:,n)) +...
        rho_tv*(Htv'*Htv) + (rho_ss+rho_ps+epsilon)*eye(nx*nx));
    if IsNorm == 1  % V2 normalisation: max to 1
        coef(n)         = max(options.V2(:,n));
        Normalize(n) 	= sum(sqrt(abs(options.V2(:,n))) );
        options.V2(:,n) = (options.V2(:,n) ./coef(n));
    else
        Normalize(n) 	= 1;
    end
end
if rho_ss>0, Mspec = (1/rho_ss)*inv( Ht*Ht' + eye(Nwvl) ); end
options.Normalize  = Normalize;

Fx          = yc;
sol         = zeros(nx,nx,Nwvl);

global loop;
loop        = 1;

h1          = figure;
set(h1,'ButtonDownFcn','MyStopFct;');
h2          = figure;

ind         = 0;

% pause;

while loop  == 1
    ind     = ind + 1;
    
    % update of yc from V2
    u       = yc(:) + tau_pwc(:)/rho_y;
    yp      = ProxWV2(options,u);
    yp      = reshape(yp,Nbase,Nwvl);
    % update of yc from Xi
    u       = yc(:) + tau_xic(:)/rho_y;
    yx      = GradphiWMF(options,u);
    yx      = reshape(yx,Nbase,Nwvl);
    % mean of lagrange multipliers and complexe visibilities estimate
    ye      =  ys;
    yc      = ( yp+yx+Fx +(tau_xc - (tau_pwc+tau_xic))./rho_y  )./3;
    ys      = yc;
    % update of x_m
     fprintf('\n Number of Wavelength: %d \n', Nwvl);
    for n   = 1 : Nwvl, 
        fprintf('|%d', n );
        tau_x   = [real(tau_xc(:,n)) ;imag(tau_xc(:,n)) ];
        yrused  = [real(yc(:,n));imag(yc(:,n))];
        
        x1      = Fr3D(:,:,n)'*(rho_y*yrused -tau_x);
        x(:,n)  = M(:,:,n)*( x1 + Htv'*(rho_tv*z(:,n)-tau_tv(:,n)) +...
            (rho_ss*v(:,n)-tau_v(:,n)) + (rho_ps*w(:,n)-tau_w(:,n)) );
        
        X           = sol;
        sol(:,:,n)  = reshape(x(:,n),nx,nx);
        Fx(:,n)     = F3D(:,:,n)*x(:,n) ;
    end
    % update of z
    HX      = Htv*x;
    if rho_tv>0
        u       = HX+tau_tv/rho_tv;
        z       = Uprox_lp(u,lambda_tv/rho_tv,p_tv);
    end
    % update of v
    if rho_ss>0
        v       = ( (rho_ss*r - tau_r)*Ht' + rho_ss*x + tau_v)*Mspec;
    end
    vHt     = v*Ht;
    if rho_ss>0
        % update of r
        u       = vHt + tau_r/rho_ss;
        r       = Uprox_lp(u,lambda_ss/rho_ss,p_ss);
    end
    % update of w
    if rho_ps>0
        u       = x + tau_w./rho_ps;
        w 		= max(max(0,u).*reshape(mask3D,[],Nwvl) - lambda_L1,0);
    end
    % update of tau
    tau_pwc = tau_pwc+rho_y *(yc - yp);
    tau_xic = tau_xic+rho_y *(yc - yx);
    tau_xc  = tau_xc +rho_y *(Fx - yc);
    tau_tv  = tau_tv +rho_tv*(HX  - z);
    tau_v   = tau_v  +rho_ss*(x   - v);
    tau_w   = tau_w  +rho_ps*(x   - w);
    if rho_ss>0,
        tau_r   = tau_r  +rho_ss*(vHt - r);
    end
    
    % stop criteria and plot
    n1      = norm( sol(:)-X(:) ,2);
    n2      = norm(rho_y*F'*(ye(:)-ys(:)),2);    
    ok      = (n1>options.eps1)|(n2>options.eps2);    
    loop    = (loop*ok)&(ind <= NbIter);
    
    residual(ind,:)     = [n1,n2];
    AllSol(:,:,:,ind)   = sol;
    options.MasqueLgr   = reshape(double(w>0),[nx nx Nwvl]);
    AllPos(:,:,:,ind)   = options.MasqueLgr;
    
    options.residual    = residual;
    options.sol         = sol;
    
    [h1,h2]             = DrawPlot(options,h1,h2);
    
    if IsSaveIt == 1
        save(['SaveOptions_' num2str(ind) '.mat'],'-struct','options');
    end
    
end
%% post processing
if IsNorm == 1
    for n       = 1 : options.Nwvl
        xsol        = sol(:,:,n);
        xsol        = .5.*xsol./...
               sum( abs(options.F3D(:,:,n)*xsol(:))).*options.Normalize(n);
        sol(:,:,n)  = xsol;
        yc(:,n)     = options.F3D(:,:,n)*xsol(:);
    end
    
    for ind         = 1 : size(AllSol,4)
        for n           = 1 : options.Nwvl
            xsol        = AllSol(:,:,n,ind);
            xsol        = .5.*xsol./...
               sum( abs(options.F3D(:,:,n)*xsol(:))).*options.Normalize(n);
            AllSol(:,:,n,ind)  = xsol;
        end
    end
end
options.yc              = yc;
options.IterationIndex  = ind;

if IsAllSave == 1 
options.AllSol          = AllSol;
options.Allpos          = Allw;
save(['SaveOptionsAll_' num2str(ind) '.mat'],'-struct','options');
end
%% save
if IsSave == 1
    save('PAINTER_Workspace.mat','-mat');
end

