function [H,HT3,HDP] = Ph2PhDiff(Closure_index,Nwvl,Nbas,T3,DP)
% generate the matrix H which convert phases to Closure phases (referenced
% in the index "Closure_index" and differential phases referenced in the
% index "DiffPhi_index". It is assumed that the data exists for all the
% "Nwvl" wavelength (such as all phases difference for all wavelength)
%
% The phases vector is the vectorisation of a matrix Y with Number of bases 
% row and Number of wavelength column.
%
% Closure and DiffPhi _index are the index (in term of sample) used for 
% the first wavelength, 
% exemple if Y = [y_1,y_2,y_3] for each wavelength then 
% then Closure_index [1,2,3] results in p_1+p_2-p_3 with p the phase of y
%
% Closure_index is NT3losure * 3
% DiffPhi_index is NDiff * 2
% H is [(NT3losure+NDiff)*Nwvl] * [Nbasase*Nwvl]
% HT3 closure only
% HW2 DiffPhi only
if nargin <2, Nwvl = 1; Nbas = max(Closure_index(:)); end
if nargin <3,           Nbas = max(Closure_index(:)); end
if nargin <4,                                         end
H       = [];
% The phases closure
if  T3 == 1;
NT3     = size(Closure_index,1);
rowt    = [(1:NT3*Nwvl);(1:NT3*Nwvl);(1:NT3*Nwvl)]; 
Colt    = Closure_index'; Colt = Colt(:); Colt = repmat(Colt,1,Nwvl)+ repmat((0:Nwvl-1)*Nbas,3*NT3,1) ; 
Valuet  = repmat([1;1;-1],NT3*Nwvl,1);
H       = sparse(rowt(:),Colt(:),Valuet(:),NT3*Nwvl,Nbas*Nwvl);
HT3     = H;
else
HT3     = [];    
end
% Differential Phases operator on phases
if  DP == 1;
rowb    = [(1:Nbas*(Nwvl-1));(1:Nbas*(Nwvl-1))]; 
Colb    = [repmat((1:Nbas),1,Nwvl-1);Nbas+1:Nbas*Nwvl];
Valueb  = repmat([1;-1],Nbas*(Nwvl-1),1);
H       = sparse(rowb(:),Colb(:),Valueb(:),Nbas*(Nwvl-1),Nbas*Nwvl);
HDP     = H;
else
HDP     = [];
end
% All index
if T3*DP == 1
H       = sparse([rowt(:);NT3*Nwvl+rowb(:)],[Colt(:);Colb(:)],[Valuet(:);Valueb(:)],Nbas*(Nwvl-1) + NT3*Nwvl,Nbas*Nwvl);
end