function yc = ProxWV2(options,yt)
% function yc = proxV2(P,rho,yt)
% proximal operator for estimation of complexe visibility from squared
% visibility
%%%%% in : STRUCT contains
% P vector of observed V2
% weightP vector of estimated V2 error, default: ones vector
% rho parameter
% yt vector of last estimated complex visibility
%%%%% out :
% yc vector of estimated complex visibility
if isfield(options,'V2')~=1,    error(' No V2 data '); end
if isfield(options,'alpha')~=1, warning(' No Prox parameter (default 1)');      options.alpha = 1; end
if isfield(options,'V2err')~=1, warning(' No V2 error bar (default 1)');        options.V2err = ones(size( options.V2 )); end
if isfield(options,'rho_y')==0, warning(' No ADMM parameter (default rho = 1)');options.rho_y = 1;    end

P       = options.V2(:);
W       = options.V2err(:);
rho     = options.rho_y;
alpha   = options.alpha;

ny      = length(yt);
yc      = zeros(ny,1);
Theta   = angle(yt);

for n   = 1:ny, 
% 	sol = cubicroots(4*alpha/W(n),0,rho-4*P(n)*alpha/W(n),-rho*abs(yt(n)));
    rhoc= rho * W(n)/(4*alpha);
    sol = cubicroots(1,0,rhoc -P(n),-rhoc*abs(yt(n)));
	sol = sol(sol>0);
    if isempty(sol)==1, sol = 0; end
	fsol= (alpha/W(n))*(P(n)-sol.^2).^2 + .5.*rho*(sol-abs(yt(n))).^2;
	[~,ind] = min(fsol); 
	yc(n,1)   = sol(ind)*exp(1i*Theta(n));    
end
if isfield(options,'printcrit') == 1 
if options.printcrit == 1
fprintf('\n MODULUS ');
fprintf('\n p1 = || P - |y|^2 ||^2_(2,W)    \t=%g \t, alpha*p1= %g'  ,norm( (P-abs(yc).^2)./W )/sqrt(ny),alpha*norm((P-abs(yc).^2)./W )/sqrt(ny));
fprintf('\n p2= .5*|| |y| - |ytilde|  ||^2_2\t=%g \t, rho/2*p2= %g\n',norm(.5*(abs(yc)-abs(yt)))/sqrt(ny),rho *norm(.5*(abs(yc)-abs(yt)))/sqrt(ny));
end
end
%%
% CUBICROOTS - finds real valued roots of cubic
% Arguments:
%     a, b, c, d - coeffecients of cubic defined as
%                  ax^3 + bx^2 + cx + d = 0
% Returns:
%     root   - an array of 1 or 3 real valued roots
% Reference:  mathworld.wolfram.com/CubicFormula.html
% Code follows Cardano's formula
% Copyright (c) 2008 Peter Kovesi
% School of Computer Science & Software Engineering
% The University of Western Australia
% pk at csse uwa edu au
% http://www.csse.uwa.edu.au/

function root = cubicroots(a,b,c,d)
    
    if abs(a) < eps
        error('this is a quadratic')
    end
    
    % Divide through by a to simplify things
    b = b/a; c = c/a; d = d/a;
    bOn3 = b/3;
    
    q = (3*c - b^2)/9;
    r = (9*b*c - 27*d - 2*b^3)/54;
    discriminant = q^3 + r^2;
    
    if discriminant >= 0        % We have 1 real root and 2 imaginary
        s = realcuberoot(r + sqrt(discriminant));   
        t = realcuberoot(r - sqrt(discriminant));
    
        root = s + t - bOn3;    % Just calculate the real root
        
    else                        % We have 3 real roots

        % In this case (r + sqrt(discriminate)) is complex so the following
        % code constructs the cube root of this complex quantity
        
        rho = sqrt(r^2 - discriminant); 
        cubeRootrho = realcuberoot(rho); % Cube root of complex magnitude
        thetaOn3 = acos(r/rho)/3;        % Complex angle/3
        
        crRhoCosThetaOn3 = cubeRootrho*cos(thetaOn3);
        crRhoSinThetaOn3 = cubeRootrho*sin(thetaOn3);   

        root = zeros(1,3);
        root(1) = 2*crRhoCosThetaOn3 - bOn3;
        root(2) =  -crRhoCosThetaOn3 - bOn3 - sqrt(3)*crRhoSinThetaOn3;
        root(3) =  -crRhoCosThetaOn3 - bOn3 + sqrt(3)*crRhoSinThetaOn3;
    end    

    
%-----------------------------------------------------------------------------
% REALCUBEROOT - computes real-valued cube root
%
% Usage:   y = realcuberoot(x)
%
% In general there will be 3 solutions for the cube root of a number. Two
% will be complex and one will be real valued.  When you raise a negative
% number to the power 1/3 in MATLAB you will not, by default, get the real
% valued solution. This function ensures you get the real one

function y = realcuberoot(x)
    
    y = sign(x).*abs(x).^(1/3);
    
% Permission is hereby granted, free of charge, to any person obtaining a copy
% of this software and associated documentation files (the "Software"), to deal
% in the Software without restriction, subject to the following conditions:
% 
% The above copyright notice and this permission notice shall be included in 
% all copies or substantial portions of the Software.
%
% The Software is provided "as is", without warranty of any kind.

% Nov 2008
    