function DATA = ReadOIFITS(dataname,datapath)
%%
% [Struct] = OIfitsReader(string,string)
%
% OIFITS Interface for MATLAB
% oifits file must be at 'datapath filesep dataname.fits' or just at 'dataname'
% DATA is a structure wich contains all OIFITS DATA (with complete header information)
%% Author
% Antony SCHUTZ 2012-2014
% Lab: J-L Lagrange CNRS OCA
% ANR POLCA
% aschutz@oca.eu - antonyschutz@gmail.com
% Version: V2.0
%
%% Check nargin
if nargin           == 2,
    pathname        = fullfile(datapath,dataname);
else
    pathname        = dataname;
end
%% can be changed if OIFITS standard changes
% Line length of headers
LineLength      = 80;
%% Main
fid             = fopen(pathname,'r','ieee-be');
DATAInfo        = fread(fid, 'uchar=>char','ieee-be');
ToHeader        = reshape(DATAInfo,LineLength,[])';
position        = ftell(fid);

% Create Header for all fields (also repetetive fields)
[DATAFields,indBeg,indEnd] = MakeHeader(ToHeader);
% Create Table with DATA Fields
DATA = MakeTable(DATAFields,fid,indBeg,indEnd,LineLength,position);

function DATA   = FillFieldAndHeaders(TableInfo,DATAFields)
DATA            = {};
for n   = 1 : length(TableInfo)
    eval(['DATA.' char(TableInfo{n}.ParentNa) '={};']);
end

List    = fieldnames(DATA);
List2   = fieldnames(DATAFields);
for   n = 1 : length(List)
    fname   = char(List{n});
    cpt     = 0;
    for   m = 1 : length(List2)
        ftest   = char(fieldnames(eval(['DATAFields.'  char(List2{m} )])));
        if strcmp(fname,ftest)==1
            cpt = cpt + 1;
            if cpt ==1, cpta = []; else cpta = cpt-1; end
            stringC = sprintf('DATA.%s.HEADER%d=DATAFields.%s.%s.HEADER;',fname,cpta,char(List2{m}),ftest);
            eval(stringC);
        end
    end
end

%% Create Table
function DATA = MakeTable(DATAFields,fid,indBeg,indEnd,LineLength,position)
% 0 - find information about table in DATA, per field:
% Line length
% number of table
% table name and optionally unity
% data format per table

indBeg      = [indBeg(2:end) position/LineLength];
TableInfo   = FindTableinfo(DATAFields);
% 1 - fill DATA Structure with non repetitive field names
DATA        = FillFieldAndHeaders(TableInfo,DATAFields);

% 2 - Create structur DATA
% for all Binary table
for m = 1 : length(TableInfo)
    % for all table in the field
    if TableInfo{m}.NbFields ~= 0, % if table exists
        % create structure for all fields
        % BytesNb = TableInfo{m}.LineBiNb;
        LineNb  = TableInfo{m}.NumbLine;
        Parent  = char(TableInfo{m}.ParentNa);
        % fill from the good line, remove unusefull space
        inddeb  = RemoveSpace(LineLength,fid,indEnd(m),indBeg(m));
        
        % put the position
        fseek(fid, inddeb ,'bof');
        for line    = 1:LineNb
            % for each line fill the table with the correct format
            for n       = 1:TableInfo{m}.NbFields
                % field
                Table   = TableInfo{m}.name{n};
                Format  = TableInfo{m}.form{n};
                [prec type flag] = DetPrecType(Format);
                data    = fread(fid,prec,type);
                data    = data(:)';
                N       = length(data);
                DataParentTable = ['DATA.' Parent '.' Table];
                
                if isfield( eval(['DATA.' Parent]), Table ) ~= 1;
                    StringT = ['DATA.' Parent '.' Table '=[];'];
                    eval(StringT);
                    switch flag
                        case '',
                            StringC = [DataParentTable '= data;'];
                        case 'Complex',
                            StringC = [DataParentTable '= data(1:2:end) + 1i.*data(2:2:end);'];
                        case 'Long',
                            clear data2;
                            for nn =1 : N,
                                data2{1,nn}=data(nn);
                            end
                            StringC = [DataParentTable '= data2;'];
                        case 'String',
                            StringC = [DataParentTable '= data ;'];
                        otherwise, error('Invalid data format');
                    end
                else
                    switch flag
                        case '',
                            StringC = [DataParentTable '=[' DataParentTable ';data];'];
                        case 'Complex',
                            StringC = [DataParentTable '=[' DataParentTable ';data(1:2:end) + 1i.*data(2:2:end)];'];
                        case 'Long',
                            clear data2;
                            for nn =1 : N,
                                data2{1,nn}=data(nn);
                            end
                            StringC = [DataParentTable '=[' DataParentTable ';data2];'];
                        case 'String',
                            StringC = [DataParentTable '=[' DataParentTable ';data];'];
                        otherwise, error('Invalid data format');
                    end
                end
                % disp('0');
                % if strcmp(StringC,'DATA.OI_ARRAY.STAXYZ=[DATA.OI_ARRAY.STAXYZ;data];') == 1;
                %     disp('1');
                %     Parent
                %     Table
                %     keyboard;
                % end
                %
                % disp('2'); keyboard;
                eval(StringC);
                clear data data2 StringC;
                
                
                % switch flag
                %     case '',
                %         StringC = ['DATA.' Parent '.' Table '=[' 'DATA.' Parent '.' Table ';data];'];
                %     case 'Complex',
                %         StringC = ['DATA.' Parent '.' Table '=[' 'DATA.' Parent '.' Table ';data(1:2:end) + 1i.*data(2:2:end)];'];
                %     case 'Long',    N = length(data);
                %         clear data2;
                % keyboard;
                %         for nn =1 : N,
                %             data2{1,nn}=data(nn);
                %         end
                %          StringC = ['DATA.' Parent '.' Table '=[' 'DATA.' Parent '.' Table ';data2];'];
                % keyboard;
                %     case 'String',
                %         StringC = ['DATA.' Parent '.' Table '=[' 'DATA.' Parent '.' Table ';data];'];
                %     otherwise, error('Invalid data format');
                % end
                %         eval(StringC);
                %         clear data;
                %         clear data2;
            end
            
        end
    end
end

% table information: name, length, format ...
function TableInfo  = FindTableinfo(DATA)
[NbXtension,~]      = size(struct2cell(DATA));
% Table Name
TableList           = fieldnames(DATA);
TableInfo           = {};
% Table Info and data structur
for m                   = 1 : NbXtension
    % parent name
    % Tabletmp                = eval([ TableList{m} '.%s'
    SubField                = eval([ 'fieldnames(DATA.' TableList{m} ');']);
    TableInfo{m}.ParentNa   = SubField;
    % open header information of table
    % tmp                     = getfield(eval(['DATA.' TableList{m} '.' cell2mat(SubField)]),'HEADER');
    tmp                     = getfield(eval(['DATA.' TableList{m} '.' char(SubField)]),'HEADER');
    % verify if is there is associated tables to the header
    test = isfield(tmp,'TFIELDS');
    if test == 1 % search data info
        % number of bytes per row
        TableInfo{m}.LineBiNb  = str2double(tmp.NAXIS1{1});
        % number of line
        TableInfo{m}.NumbLine  = str2double(tmp.NAXIS2{1});
        % number of table
        NbFields            = str2double(tmp.TFIELDS{1});
        TableInfo{m}.NbFields  = NbFields;
        % information per table
        for n = 1 : NbFields
            TableInfo{m}.name{n}   = eval(sprintf('tmp.TTYPE%d{1}',n));
            TableInfo{m}.form{n}   = eval(sprintf('tmp.TFORM%d{1}',n));
            if isfield(tmp,sprintf('TUNIT%d',n))==1
                TableInfo{m}.unit{n}   = eval(sprintf('tmp.TUNIT%d{1}',n));
            else
                TableInfo{m}.unit{n}   = '';
            end
        end
    else
        TableInfo{m}.NbFields  = 0;
        TableInfo{m}.LineBiNb  = 0;
        TableInfo{m}.NumbLine  = 0;
        TableInfo{m}.name      = {};
        TableInfo{m}.unit      = {};
        TableInfo{m}.form      = {};
    end
end

% Create straucture with fields
function DATAFields = CreateField(NomHeader, indBeg,indEnd,List )
sep                 = {'=',' / '};
DATAFields          = {};

for           m = 1 : length(indBeg)
    Begin           = indBeg(m);
    End             = indEnd(m);
    TableName       = List{m};
    
    Reduced         = NomHeader(1+Begin:End-1,:);
    
    string0         = sprintf('DATAFields.Field%d.%s.HEADER = {};',m,TableName);
    eval(string0);
    cpt             = 0;
    
    for n       = 1 : size(Reduced,1);
        
        tmp     = Reduced(n,:);
        ind1    = strfind(tmp,sep{1});
        ind2    = strfind(tmp,sep{2});
        
        %     if n == 16, keyboard; end
        
        if strcmp(tmp(1:7),'COMMENT') == 1
            cpt     = cpt + 1;
            ind     = strfind(tmp,''''); tmp(ind)=[];
            string1 = sprintf('DATAFields.Field%d.%s.HEADER.COMMENT%d{1}=''%s'';',m,TableName,cpt, tmp(11:end));
            eval(string1);
            
        else
            FieldName       = sscanf(tmp(1:ind1-1),'%s');
            if isempty(ind2)==1
                FieldValue      = sscanf(tmp(ind1+1:end),'%s');
            else
                FieldValue      = sscanf(tmp(ind1+1:ind2-1),'%s');
            end
            if numel(FieldValue)~=0
            if double(FieldValue(1)) == 39, FieldValue = FieldValue(2:end-1 ); end % ' char pb
            ind     = strfind(FieldName,'-') ; if isempty(ind)~=1, FieldName(ind) = '_'; end
            ind     = strfind(FieldName,' ') ; if isempty(ind)~=1, FieldName(ind) = '_'; end
            FieldComment    = tmp(ind2+2:end);
            ind     = strfind(FieldComment,'''') ; if isempty(ind)~=1, FieldComment(ind) = []; end
            string1 = sprintf('DATAFields.Field%d.%s.HEADER.%s{1}=''%s'';',m,TableName, FieldName , num2str(FieldValue));
            string2 = sprintf('DATAFields.Field%d.%s.HEADER.%s{2}=''%s'';',m,TableName, FieldName , FieldComment);
            eval(string1);
            eval(string2);
            end            
        end
    end
end

%% Create Header in struct DATA
function [DATAFields,indBeg,indEnd] = MakeHeader(ToHeader)

% BEGIN and END  index of each headers
indBeg      = [0 FindBegTableIndex(ToHeader)];
indEnd      = FindEndTableIndex(ToHeader);
% find Table name otherwise it is Primary
sep         = {'=',' / '};
cp          = 1;
% check if Primary exists and if there is no blank line before
ok          = 1;
n           = 0;
while ok    ==1;
    n           = n + 1;
    tmp         = ToHeader(n,:);
    if mean(double(tmp)) ~= 32
        if strcmp(tmp(1:8),'XTENSION')~=1
            List{cp}= 'Primary';
            ok      = 0;
        end
    end
    if n> indEnd(1); ok =0; end
end

for n   = 1 : length(indBeg)
    for m   = 1+indBeg(n):indEnd(n)
        tmp     = ToHeader(m,:);
        test    = strfind(tmp,'EXTNAME');
        if isempty(test)~=1
            ind1    = strfind(tmp,sep{1});
            ind2    = strfind(tmp,sep{2});
            name    = sscanf(tmp(ind1+1:ind2-1),'%s');
            cp      = cp  + 1;
            List{cp}= name(2:end-1);
        end
    end
end
% test is the same "list name" appears several times, if yes
% concatenates data -> use a flag
DATAFields     = CreateField(ToHeader,indBeg,indEnd,List);

% find the begining index of each table
function indBeg = FindBegTableIndex(ToHeader)
cpt     = 0;
keyword = 'XTENSION= ''BINTABLE''';
for n = 1 : size(ToHeader,1)
    if strcmp(ToHeader(n,1:length(keyword)),keyword)==1
        cpt          = cpt + 1;
        indBeg(cpt)  = n;
    end
end
% find the end index of each table
function indEnd = FindEndTableIndex(ToHeader)
cpt     = 0;
keyword = 'END';
for n = 1 : size(ToHeader,1)
    if strcmp(ToHeader(n,1:length(keyword)),keyword)==1
        cpt          = cpt + 1;
        indEnd(cpt)  = n;
    end
end

%% Remove blank line between header and tables
function inddeb = RemoveSpace(LineLength,fid,indEndn,indBegn)
fseek(fid,0,'bof');
TmpDATA     = fread(fid, 'uchar=>char','ieee-be');
NomHeader   = reshape(TmpDATA,LineLength,[])';
TmpDATA     = NomHeader((indEndn+1):(indBegn-1),:);

for n   = 1 : size(TmpDATA,1)
    tmp(n)  = mean(double(TmpDATA(n,:) )) ==32;
end
inddeb = (indEndn + sum(tmp))*LineLength;


%% FORMAT and Length definition from fits info
function [prec type flag] = DetPrecType(datainfo)
datainfo    = sscanf(datainfo,'%s');
prec        = sscanf(datainfo,'%d');
if isempty(prec) ==1, prec = 1; end
datatype    = datainfo(end);
switch datatype
    case 'I', type = 'int16';           flag = '';
    case 'D', type = 'double';          flag = '';
    case 'E', type = 'float';           flag = '';
    case 'A', type = 'uchar=>char';     flag = 'String';
    case 'L', type = 'uchar=>char';     flag = 'Long';
    case 'C', type = 'float';           flag = 'Complex';
        prec = 2*prec; % real and imag part
    otherwise, error('error of Format');
end
