function [options,F,F3D,Fr,Fr3D] = makeF(options)
% create Non Uniform Fourier Matrix for an image of size nx*ny
% the sampling (spatiale frequency) is given by UV in radian, FOV is the
% field of view in arcsecond so that: |UV*coef|<0.5 and coef is the field of
% view in radian
%
% strut contain UV plan information (Nbase*Nwavelength)
if isfield(options,'U')~=1, error(' No U (UVplan) information '); end
if isfield(options,'V')~=1, error(' No V (UVplan) information '); end

U           = options.U;
V           = options.V;

DegRad      = 2*pi/360.0;
RadArcsec   = 3600.0/DegRad;
coef        = options.FOV/RadArcsec;
U           = U  .* coef; 
V           = V  .* coef; 
[Nbase,Nwvl]= size(U);
F3D         = zeros(Nbase,options.nx*options.nx,Nwvl);
for     n   = 1 : Nwvl
        u   = U(:,n);
        v   = V(:,n);            
        tmpF        = NuDFT([u(:),v(:)],options.nx,options.nx);   
        F3D(:,:,n)  = tmpF;        
end
F       = NuDFT([U(:),V(:)],options.nx,options.nx);   
Fr      = [real(F);imag(F)];
Fr3D    = [real(F3D);imag(F3D)];

options.F    = F;
options.Fr   = Fr;
options.F3D  = F3D;
options.Fr3D = Fr3D;