%% warning declaration
if isfield(options,'type')~=1    
options.type = '3D';
end
    
if isfield(options,'nx')~=1    
    nx   = 40;     
else
    nx   = options.nx;
end
Htv             = getH(nx,nx);
options.Htv     = Htv;
disp([' Image size: ' num2str(nx) 'px, Default: 40px ']);

if isfield(options,'Nwvl')~=1    
    error(' Wavelength information missing');
else
    Nwvl   = options.Nwvl;
    disp([' ' num2str(Nwvl) ' Wavelengths ' ]);
    % defines the subplot
    [options.SubColumn,options.SubRow] = SubIndexPlot(Nwvl);
    options = getHSpec(options);
    Ht      = options.Ht;
end


if isfield(options,'Nbase')~=1    
    error(' No Base information missing');
else
    Nbase   = options.Nbase;
end


if isfield(options,'FontSize')~=1    
    FontSize   = 12;
else
    FontSize   = options.FontSize;
end

if isfield(options,'FontWeight')~=1    
    FontWeight   = 'bold';
else
    FontWeight   = options.FontWeight;
end

if isfield(options,'IsAllSave')~=1
    warning('Results are not saved, default 0');
    IsAllSave   = 0;
else
    IsAllSave   = options.IsAllSave;
end

if isfield(options,'IsSave')~=1
    warning('Results are not saved, default 0');
    IsSave   = 0;
else
    IsSave   = options.IsSave;
end

if isfield(options,'IsSaveIt')~=1
    warning('Results are not saved during loop, default 0');
    IsSaveIt   = 0;
else
    IsSaveIt   = options.IsSaveIt;
end

if isfield(options,'IsNorm')~=1
    warning('Channels are not normalized, default 0');
    IsNorm   = 0;
else
    IsNorm   = options.IsNorm;
end

if isfield(options,'plotScale')~=1
    warning('plot Style, default is Lin (linear scale)');
    options.plotScale   = 'Lin';
end



if isfield(options,'rho_y')~=1
    warning('No ADMM parameters for complexe visibility, default 1');
    rho_y   = 1;
else
    rho_y   = options.rho_y;
end

if isfield(options,'lambda_L1')~=1
    lambda_L1   = 0;
else
    lambda_L1   = options.lambda_L1;
end

if isfield(options,'rho_ps')~=1
    warning('No ADMM parameters for positivity, default 0, no positivity');
    rho_ps      = 0;
else
    disp(' Positivity contraint ');
    rho_ps      = options.rho_ps;
end

if isfield(options,'rho_ss')~=1
    warning('No ADMM parameters for spectral smoothness, default 0, no S.Smoothness');
    rho_ss      = 0;
else
    disp(' Spectral Smoothness contraint ');
    rho_ss      = options.rho_ss;
    p_ss        = options.p_ss;
    lambda_ss   = options.lambda_ss/nnz(Ht);
end

if isfield(options,'rho_tv')~=1
    warning('No ADMM parameters for Total Variation, default 0, no TV');
    rho_tv      = 0;
else
    disp(' Total Variation contraint ');
    rho_tv      = options.rho_tv;
    p_tv        = options.p_tv;
    lambda_tv   = options.lambda_tv/nnz(Htv);
end

if isfield(options,'epsilon')~=1
    warning('No ADMM parameters for ridge, default 0, no Ridge');
    epsilon     = 0;
else
    disp(' Ridge contraint ');
    epsilon     = options.epsilon;
end

%% FOURIER MATRIX
if isfield(options,'F')~=1
    error(' No Non-Uniform DFT Matrix');
else
    F           = options.F;
    Fr          = options.Fr;
    F3D         = options.F3D;
    Fr3D        = options.Fr3D;
end

if isfield(options,'F3DG')~=1
    if isfield(options,'CompVisInit3D')~=1
        if isfield(options,'xinit3D')~=1
            warning(' No complexe visibility given and no first estimate, default 0 background');
            xinit3D     = zeros(nx,nx,Nwvl);
        else
            disp(' reconstruction using object initialization ');
            xinit3D     = options.xinit3D;
        end
        yc          = zeros(Nbase,Nwvl);
        for n = 1 : Nwvl
			
			% first estimate normalization
			x3D         = xinit3D(:,:,n) ;
            yc(:,n)     = F3D(:,:,n)*x3D(:);
        end
    else
        disp(' reconstruction using Complexe Visibilities initialization');
        yc          = options.CompVisInit3D;
    end
end

if isfield(options,'mask3D')~=1
    mask3D      = ones(options.nx,options.nx,options.Nwvl);
else
    if strcmp(options.type,'3D') == 1
    disp(' 3D support contraint ');
    end
    mask3D      = options.mask3D;
end

if isfield(options,'mask')~=1
    mask      = ones(options.nx,options.nx);
else
    if strcmp(options.type,'2D') == 1
    disp(' 2D support contraint ');
    end
    mask      = options.mask;
end
%% Warning Affichage and stop
if isfield(options,'FOV')~=1
    indpix = 1 : nx; LABELS = ' pixels '; 
	options.indpix = indpix;
else
    indpix = linspace(-options.FOV/2,options.FOV/2,nx);
	options.indpix = indpix;
    options.LABELS = ' FoV units (as)';
    LABELS = ' FoV units (as)';
end 

if isfield(options,'NbIter')~=1
    NbIter = Inf;
else
    fprintf(' Maximum Number of Iteration: %d', options.NbIter );
    NbIter = options.NbIter;
end

if isfield(options,'eps1')~=1
    warning('No ADMM Primal residual stopping criteria, default 1e-3');
    eps1 = 1e-3;
else
    eps1 = options.eps1;
end

if isfield(options,'eps2')~=1
    warning('No ADMM Dual residual stopping criteria, default 1e-3');
    eps2 = 1e-3;
else
    eps2 = options.eps2;
end
%% Warning, error and variables setting for grouped data
if isfield(options,'F3DG')==1
    F3DG    = options.F3DG;
    Fr3DG   = options.Fr3DG;
    mask3D  = ones(options.nx,options.nx,options.NwvlGroup);
    if isfield(options,'CompVisInit3D')~=1
        if isfield(options,'xinit3D')~=1
            warning(' No complexe visibility given and no first estimate, default normalized background');
            xinit3D     = ones(nx,nx,options.NwvlGroup)/nx.^2/options.NwvlGroup;
        else
            xinit3D     = options.xinit3D;
            for n = 1 : options.NwvlGroup
                x3D             = xinit3D(:,:,n) ;
                options.yc{n}   = F3DG{n}*x3D(:);
            end
            % ungroup data
            options         = GroupData(options,0);
            yc              = options.ycv;
        end
    else
        disp(' reconstruction using Complexe Visibilities initialization');
        yc          = options.CompVisInit3D;
    end
end